//----------------------------------------------------------------------------
// 
//	file: inputlist.cc
// 
//	function: maintain list of input spec and return it when prompted 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Thu Feb  4 00:16:58 CST 1999
// 
//	$Revision: 1.2 $ $Date: 2001/11/30 20:24:15 $
//	$Log: inputlist.cc,v $
//	Revision 1.2  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.1  1999/02/09 07:42:25  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------

#include "inputlist.h"

//----------------------------------------------------------------------------
//	inputlist() 
//	  - constructor
//----------------------------------------------------------------------------
inputlist::inputlist(int max_specs, geninput *input_obj){

  // 1. max number of specs to allow
  inputlist::max_specs = max_specs;

  // 2. geninput object to call when add_inputs() is invoked
  inputlist::input_obj = input_obj;

  // 3. allocate table for the input specs
  //	  format:
  //		table[spec_num][arg_num]
  table = new IL_Data* [max_specs];
  for (int i=0; i<max_specs; ++i) {
	table[i] = new IL_Data [5];
  }

  type_table = new INP_TYPE [max_specs];

  if (!type_table) {
	error_exit("inputlist::inputlist","error in malloc");
  }

  // 4. reset num_specs;
  num_specs = 0;

}


//----------------------------------------------------------------------------
//	~inputlist() 
//	  -  destructor
//----------------------------------------------------------------------------
inputlist::~inputlist() {

  // deallocate tables
  
  for (int i=0; i<max_specs; ++i) {
	delete [] table[i];
  }
  delete [] table;

  delete [] type_table;

}

//----------------------------------------------------------------------------
//	add_spec() 
//	  - add input spec to the table
//----------------------------------------------------------------------------
void  inputlist::add_spec(int cx, int cy, int radius){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_SQR;

   table[num_specs][0].i 	= cx;
   table[num_specs][1].i 	= cy;
   table[num_specs][2].i 	= radius;

   num_specs ++;

}
//----------------------------------------------------------------------------
void  inputlist::add_spec(int cx, int cy, int width, int height, REAL angle){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_ORI_BAR;

   table[num_specs][0].i 	= cx;
   table[num_specs][1].i 	= cy;
   table[num_specs][2].i 	= width;
   table[num_specs][3].i 	= height;
   table[num_specs][4].r 	= angle;

   num_specs ++;

}
//----------------------------------------------------------------------------
void  inputlist::add_spec(int cx, int cy, REAL sigma){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_BLOB;

   table[num_specs][0].i 	= cx;
   table[num_specs][1].i 	= cy;
   table[num_specs][2].r   = sigma;

   num_specs ++;

}

//----------------------------------------------------------------------------
void  inputlist::add_spec(int cx, int cy, REAL sigma, REAL slope){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_BLOB;

   table[num_specs][0].i 	= cx;
   table[num_specs][1].i 	= cy;
   table[num_specs][2].r   = sigma;
   table[num_specs][3].r   = slope;

   num_specs ++;

}

//----------------------------------------------------------------------------
void  inputlist::add_spec(int cx, int cy, REAL sigmax, REAL sigmay, REAL angle){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_ORI_BLOB;

   table[num_specs][0].i 	= cx;
   table[num_specs][1].i 	= cy;
   table[num_specs][2].r 	= sigmax;
   table[num_specs][3].r 	= sigmay;
   table[num_specs][4].r 	= angle;

   num_specs ++;

}
//----------------------------------------------------------------------------
void  inputlist::add_spec(int cx, int cy, int freq, REAL phase, REAL angle){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_SIN;

   table[num_specs][0].i 	= cx;
   table[num_specs][1].i 	= cy;
   table[num_specs][2].i 	= freq;
   table[num_specs][3].r 	= phase;
   table[num_specs][4].r 	= angle;

   num_specs ++;

}
//----------------------------------------------------------------------------
void  inputlist::add_spec(int imgidx){

   if (num_specs>=max_specs) {
	warning("inputlist::add_spec","max number of input specs exceeded");
	return;
   }

   type_table[num_specs] = INP_IMG;

   table[num_specs][0].i 	= imgidx;

   num_specs ++;

}

//----------------------------------------------------------------------------
//	call_add_spec() 
//	  - interface to simenv
//----------------------------------------------------------------------------
int   inputlist::call_add_spec_rect(int argc, char *argv[]) {

   if (argc != 3) return 0;

   add_spec(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]));

   return 1;

}

//----------------------------------------------------------------------------
int   inputlist::call_add_spec_ori_line(int argc, char *argv[]) { 

   if (argc != 5) return 0;

   add_spec(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]),
                atoi(argv[4]), (REAL)atof(argv[5]));
   return 1;

}

//----------------------------------------------------------------------------
int   inputlist::call_add_spec_gaus(int argc, char *argv[]) { 

   if (argc != 3) return 0;

   add_spec(atoi(argv[1]), atoi(argv[2]), (REAL)atof(argv[3]));

   return 1;

}

//----------------------------------------------------------------------------
int   inputlist::call_add_spec_ori_gaus(int argc, char *argv[]) { 

   if (argc != 5) return 0;

   add_spec(atoi(argv[1]), atoi(argv[2]), (REAL)atof(argv[3]),
                (REAL)atof(argv[4]), (REAL)atof(argv[5]));

   return 1;

}

//----------------------------------------------------------------------------
int   inputlist::call_add_spec_sin(int argc, char *argv[]) { 

   if (argc != 5) return 0;

   add_spec(atoi(argv[1]), atoi(argv[2]), atoi(argv[3]),
                (REAL)atof(argv[4]), (REAL)atof(argv[5]));

   return 1;

}

//----------------------------------------------------------------------------
int   inputlist::call_add_spec_blobs(int argc, char *argv[]) { 

   if (argc != 5) return 0;

   add_spec(atoi(argv[1]), atoi(argv[2]), 
	(REAL)atof(argv[3]), (REAL)atof(argv[4]));

   return 1;

}

//----------------------------------------------------------------------------
int   inputlist::call_add_spec_img(int argc, char *argv[]) {

   if (argc != 1) return 0;

   add_spec(atoi(argv[1]));

   return 1;

}

//----------------------------------------------------------------------------
//	add_inputs() 
//	  - dump list on geninput object
//----------------------------------------------------------------------------
void  inputlist::add_inputs() {

  // 1. first, clear input
  input_obj->clear_input();

  for (int i=0; i<num_specs; ++i) {

     switch (type_table[i]) {

	case INP_SQR:
	     input_obj->add_input(table[i][0].i, table[i][1].i, table[i][2].i);
	     break;

	case INP_ORI_BAR:
	     input_obj->add_input(table[i][0].i, table[i][1].i, table[i][2].i,
				 table[i][3].i, table[i][4].r);
	     break;

	case INP_BLOB:
	     input_obj->add_input(table[i][0].i, table[i][1].i, table[i][2].r);
  	     break;

	case INP_ORI_BLOB:
	     input_obj->add_input((int)table[i][0].i, (int)table[i][1].i, 
				 (REAL)table[i][2].r,
				 (REAL)table[i][3].r, (REAL)table[i][4].r);
  	     break;

	case INP_SIN:
	     input_obj->add_input(table[i][0].i, table[i][1].i, table[i][2].i,
				 table[i][3].r, table[i][4].r);
	     break;

	case INP_BLOBS:
	     input_obj->add_input(table[i][0].i, table[i][1].i, table[i][2].r,
				 table[i][3].r);
	     break;

	case INP_IMG:
	     input_obj->add_input(table[i][0].i);
	     break;

	default: break;

     }

  }

}

